@@ -82,6 +82,7 @@ static int tls_ex_index_ssl_opts;
 #if defined(_WIN32) || defined(_WIN64)
 #define iov_len len
 #define iov_base buf
+#define snprintf _snprintf
 #endif
 
 /**
@@ -284,7 +285,8 @@ char* SSLSocket_get_version_string(int version)
 
 	if (retstring == NULL)
 	{
-		sprintf(buf, "%i", version);
+		if (snprintf(buf, sizeof(buf), "%i", version) >= sizeof(buf))
+			buf[sizeof(buf)-1] = '\0'; /* just in case of snprintf buffer overflow */
 		retstring = buf;
 	}
 	return retstring;
@@ -500,9 +502,9 @@ void SSLSocket_terminate(void)
 
 	if (handle_openssl_init)
 	{
-		EVP_cleanup();
-		ERR_free_strings();
 		CRYPTO_set_locking_callback(NULL);
+		ERR_free_strings();
+		EVP_cleanup();
 		if (sslLocks)
 		{
 			int i = 0;
@@ -668,6 +670,24 @@ int SSLSocket_createContext(networkHandles* net, MQTTClient_SSLOptions* opts)
 	}
 #endif
 
+#if (OPENSSL_VERSION_NUMBER >= 0x010002000) /* 1.0.2 and later */
+	if (opts->protos != NULL && opts->protos_len > 0)
+	{
+		/* SSL_CTX_set_alpn_protos() returns 0 on success as opposed to the other SSL functions,
+		   so we need to flip the meaning of rc or the return code will be wrong. */
+		if ((rc = SSL_CTX_set_alpn_protos(net->ctx, opts->protos, opts->protos_len)) != 0)
+		{
+			if (opts->struct_version >= 3)
+				SSLSocket_error("SSL_CTX_set_alpn_protos", NULL, net->socket, rc, opts->ssl_error_cb, opts->ssl_error_context);
+			else
+				SSLSocket_error("SSL_CTX_set_alpn_protos", NULL, net->socket, rc, NULL, NULL);
+			rc = 0; /* report an error according to the convention in other OpenSSL functions */
+			goto free_ctx;
+		}
+		rc = 1; /* report success according to the convention in other OpenSSL functions */
+	}
+#endif
+
 	SSL_CTX_set_mode(net->ctx, SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
 
 	goto exit;
